
CSpringProxActiveXCtl::CSpringProxActiveXCtl()
{
  m_dwCurrentSafety = INTERFACESAFE_FOR_UNTRUSTED_CALLER | INTERFACESAFE_FOR_UNTRUSTED_DATA;

  Active = FALSE;

  IsoConfig = CFG_MODE_ISO_14443_A;

  Cid = 0xFF;
  Nad = 0xFF;

  LedR = 6;
  LedG = 6;
  LedY = 6;

  DeviceName[0] = '\0';

  IsoASnrLen = 0;
  IsoAAtsLen = 0;

  MifBlockDataOK = 0;
  MifSectorDataOK = 0;

  MifKeyReadLoaded = 0;
  MifKeyWriteLoaded = 0;

  LastError = 0;
}

CSpringProxActiveXCtl::~CSpringProxActiveXCtl()
{
  Close();
}



STDMETHODIMP CSpringProxActiveXCtl::get_Library(BSTR * pVal)
{
  wchar_t buffer[256];

  LastError = SPROX_GetLibraryW(buffer, 256);
  if (LastError != MI_OK)
    return Error(_T("GetLibrary failed"));

  _tcscat(buffer, _T(" [OCX v1.5]"));

  toBSTR(pVal, buffer);

  return S_OK;
}

STDMETHODIMP CSpringProxActiveXCtl::get_LastError(SHORT * pVal)
{
  *pVal = LastError;
  return S_OK;
}

STDMETHODIMP CSpringProxActiveXCtl::get_LastErrorTxt(BSTR * pVal)
{
  const wchar_t *p;

  p = SPROX_GetErrorMessageW(LastError);
  if (p == NULL)
    return Error(_T("GetErrorMessage failed"));

  toBSTR(pVal, p);
  return S_OK;
}

STDMETHODIMP CSpringProxActiveXCtl::get_Active(BYTE * pVal)
{
  *pVal = Active;
  return S_OK;
}

STDMETHODIMP CSpringProxActiveXCtl::put_Active(BYTE newVal)
{
  if (newVal == TRUE)
    return Open();
  if (newVal == FALSE)
    return Close();

  return
    Error(_T
          ("The Active property can only have the following values: 0/FALSE or 1/TRUE"));
}

STDMETHODIMP CSpringProxActiveXCtl::get_Device(BSTR * pVal)
{
  toBSTR(pVal, DeviceName);
  return S_OK;
}

STDMETHODIMP CSpringProxActiveXCtl::put_Device(BSTR newVal)
{
  Close();

  DeviceName[0] = '\0';

  if (SysStringLen(newVal))
    wcscpy(DeviceName, newVal);

  return S_OK;
}

STDMETHODIMP CSpringProxActiveXCtl::get_Firmware(BSTR * pVal)
{
  wchar_t buffer[256];

  LastError = SPROX_ReaderGetFirmwareW(buffer, 256);
  if (LastError != MI_OK)
    return Error(_T("ReaderGetFirmware failed"));

  toBSTR(pVal, buffer);

  return S_OK;
}

STDMETHODIMP CSpringProxActiveXCtl::Open(void)
{
  if (wcslen(DeviceName))
    LastError = SPROX_ReaderOpenW(DeviceName);
  else
    LastError = SPROX_ReaderOpenW(NULL);

  if (LastError != MI_OK)
    return Error(_T("ReaderOpen failed"));

  LastError = SPROX_ReaderGetDeviceW(DeviceName, sizeof(DeviceName) / sizeof(wchar_t));
  if (LastError != MI_OK)
    return Error(_T("ReaderGetDevice failed"));

  Active = TRUE;
  return S_OK;
}

STDMETHODIMP CSpringProxActiveXCtl::Close(void)
{
  LastError = SPROX_ReaderClose();
  Active = FALSE;
  return S_OK;
}

STDMETHODIMP CSpringProxActiveXCtl::put_RfField(BYTE newVal)
{
  LastError = SPROX_ControlRF(newVal);
  if (LastError != MI_OK)
    return Error(_T("ControlRF failed"));

  return S_OK;
}

STDMETHODIMP CSpringProxActiveXCtl::put_LedRed(BYTE newVal)
{
  LedR = newVal;

  LastError = SPROX_ControlLed(LedR, LedG);
  if (LastError != MI_OK)
    return Error(_T("ControlLed failed"));

  return S_OK;
}

STDMETHODIMP CSpringProxActiveXCtl::put_LedGreen(BYTE newVal)
{
  LedG = newVal;

  LastError = SPROX_ControlLed(LedR, LedG);
  if (LastError != MI_OK)
    return Error(_T("ControlLed failed"));

  return S_OK;
}

STDMETHODIMP CSpringProxActiveXCtl::put_LedYellow(BYTE newVal)
{
  LedY = newVal;

  LastError = SPROX_ControlLedY(LedR, LedG, LedY);
  if (LastError != MI_OK)
    return Error(_T("ControlLed failed"));

  return S_OK;
}

STDMETHODIMP CSpringProxActiveXCtl::put_IsoConfig(BYTE newVal)
{
  switch (newVal)
  {
    case 'A':
    case 'a':
    case 0x0A:
    case CFG_MODE_ISO_14443_A:
      IsoConfig = CFG_MODE_ISO_14443_A;
      break;

    case 'B':
    case 'b':
    case 0x0B:
    case CFG_MODE_ISO_14443_B:
      IsoConfig = CFG_MODE_ISO_14443_B;
      break;

    default:
      return
        Error(_T
              ("The IsoConfig property can only have the following values: 1 (A) or 2 (B)"));
  }

  LastError = SPROX_SetConfig(IsoConfig);

  if (LastError != MI_OK)
    return Error(_T("SetConfig failed"));

  return S_OK;
}

STDMETHODIMP CSpringProxActiveXCtl::get_IsoConfig(BYTE * pVal)
{
  *pVal = IsoConfig;
  return S_OK;
}

STDMETHODIMP CSpringProxActiveXCtl::TagSelect(void)
{
  return SelectAny();
}

STDMETHODIMP CSpringProxActiveXCtl::SelectAny(void)
{
  if (IsoConfig == CFG_MODE_ISO_14443_A)
  {
    IsoASnrLen = sizeof(IsoASnr);
    LastError = SPROX_A_SelectAny(IsoAAtq, IsoASnr, &IsoASnrLen, IsoASak);
  } else if (IsoConfig == CFG_MODE_ISO_14443_B)
  {
    LastError = SPROX_B_SelectAny(0x00, IsoBAtq);
  } else
  {
    return Error(_T("Invalid ISO 14443 mode"));
  }

  if (LastError != MI_OK)
  {
    if (LastError == MI_NOTAGERR)
    {
      // No tag found
      return Error(_T("No card found on reader's antenna"));
    } else
    {
      // Error while selecting tag
      return Error(_T("Error in card selection"));
    }
  }

  return S_OK;
}

STDMETHODIMP CSpringProxActiveXCtl::SelectIdle(void)
{
  if (IsoConfig == CFG_MODE_ISO_14443_A)
  {
    IsoASnrLen = sizeof(IsoASnr);
    LastError = SPROX_A_SelectIdle(IsoAAtq, IsoASnr, &IsoASnrLen, IsoASak);
  } else if (IsoConfig == CFG_MODE_ISO_14443_B)
  {
    LastError = SPROX_B_SelectIdle(0x00, IsoBAtq);
  } else
  {
    return Error(_T("Invalid ISO 14443 mode"));
  }

  if (LastError != MI_OK)
  {
    if (LastError == MI_NOTAGERR)
    {
      // No tag found
      return Error(_T("No card found on reader's antenna"));
    } else
    {
      // Error while selecting tag
      return Error(_T("Error in card selection"));
    }
  }

  return S_OK;
}

STDMETHODIMP CSpringProxActiveXCtl::SelectAgain(void)
{
  if (IsoConfig == CFG_MODE_ISO_14443_A)
  {
    LastError = SPROX_A_SelectAgain(IsoASnr, IsoASnrLen, IsoASak);
  } else if (IsoConfig == CFG_MODE_ISO_14443_B)
  {
    return Error(_T("Function not available"));
  } else
  {
    return Error(_T("Invalid ISO 14443 mode"));
  }

  if (LastError != MI_OK)
  {
    if (LastError == MI_NOTAGERR)
    {
      // No tag found
      return Error(_T("No card found on reader's antenna"));
    } else
    {
      // Error while selecting tag
      return Error(_T("Error in card selection"));
    }
  }

  return S_OK;
}

STDMETHODIMP CSpringProxActiveXCtl::TagHalt(void)
{
  return Halt();
}

STDMETHODIMP CSpringProxActiveXCtl::Halt(void)
{
  if (IsoConfig == CFG_MODE_ISO_14443_A)
  {
    LastError = SPROX_TclA_Halt();
  } else if (IsoConfig == CFG_MODE_ISO_14443_B)
  {
    LastError = SPROX_TclB_Halt(IsoBAtq);
  } else
  {
    return Error(_T("Invalid ISO 14443 mode"));
  }

  return S_OK;
}

STDMETHODIMP CSpringProxActiveXCtl::get_TagSnr(BSTR * pVal)
{
  return get_Uid(pVal);
}

STDMETHODIMP CSpringProxActiveXCtl::get_Uid(BSTR * pVal)
{
  wchar_t buffer[256];

  if (IsoConfig != CFG_MODE_ISO_14443_A)
    return Error(_T("Invalid ISO 14443 mode"));

  toHEX(buffer, IsoASnr, IsoASnrLen);
  toBSTR(pVal, buffer);

  return S_OK;
}

STDMETHODIMP CSpringProxActiveXCtl::get_Pupi(BSTR * pVal)
{
  wchar_t buffer[256];

  if (IsoConfig != CFG_MODE_ISO_14443_B)
    return Error(_T("Invalid ISO 14443 mode"));

  toHEX(buffer, IsoBAtq, 4);
  toBSTR(pVal, buffer);

  return S_OK;
}

STDMETHODIMP CSpringProxActiveXCtl::get_TagAtq(BSTR * pVal)
{
  return get_Atq(pVal);
}

STDMETHODIMP CSpringProxActiveXCtl::get_Atq(BSTR * pVal)
{
  wchar_t buffer[256];

  if (IsoConfig == CFG_MODE_ISO_14443_A)
  {
    toHEX(buffer, IsoAAtq, 2);
  } else if (IsoConfig == CFG_MODE_ISO_14443_B)
  {
    toHEX(buffer, IsoBAtq, 11);
  } else
  {
    return Error(_T("Invalid ISO 14443 mode"));
  }

  toBSTR(pVal, buffer);
  return S_OK;
}

STDMETHODIMP CSpringProxActiveXCtl::get_TagSak(BSTR * pVal)
{
  return get_Sak(pVal);
}

STDMETHODIMP CSpringProxActiveXCtl::get_Sak(BSTR * pVal)
{
  wchar_t buffer[256];

  if (IsoConfig != CFG_MODE_ISO_14443_A)
    return Error(_T("Invalid ISO 14443 mode"));

  toHEX(buffer, IsoASak, 1);
  toBSTR(pVal, buffer);

  return S_OK;
}

STDMETHODIMP CSpringProxActiveXCtl::put_Cid(BYTE newVal)
{
  if (newVal < 15)
  {
    Cid = newVal;
  } else
  {
    Cid = 0xFF;
  }

  return S_OK;
}

STDMETHODIMP CSpringProxActiveXCtl::get_Nad(BYTE * pVal)
{
  *pVal = Nad;
  return S_OK;
}

STDMETHODIMP CSpringProxActiveXCtl::put_Nad(BYTE newVal)
{
  Nad = newVal;
  return S_OK;
}

STDMETHODIMP CSpringProxActiveXCtl::get_Cid(BYTE * pVal)
{
  *pVal = Cid;
  return S_OK;
}

STDMETHODIMP CSpringProxActiveXCtl::TclSelect(void)
{
  if (IsoConfig == CFG_MODE_ISO_14443_A)
  {
    IsoAAtsLen = sizeof(IsoAAts);
    LastError = SPROX_TclA_GetAts(Cid, IsoAAts, &IsoAAtsLen);
  } else if (IsoConfig == CFG_MODE_ISO_14443_B)
  {
    LastError = SPROX_TclB_Attrib(IsoBAtq, Cid);
  } else
  {
    return Error(_T("SPROX: Invalid ISO 14443 mode"));
  }

  if (LastError != MI_OK)
    return Error(_T("T=CL activation failed"));

  return S_OK;
}


STDMETHODIMP CSpringProxActiveXCtl::get_TclAts(BSTR * pVal)
{
  return get_Ats(pVal);
}

STDMETHODIMP CSpringProxActiveXCtl::get_Ats(BSTR * pVal)
{
  wchar_t buffer[256];

  if (IsoConfig == CFG_MODE_ISO_14443_A)
  {
    toHEX(buffer, IsoAAts, IsoAAtsLen);
  } else if (IsoConfig == CFG_MODE_ISO_14443_B)
  {
    buffer[0] = '\0';
  } else
  {
    return Error(_T("Invalid ISO 14443 mode"));
  }

  toBSTR(pVal, buffer);
  return S_OK;
}

STDMETHODIMP CSpringProxActiveXCtl::TclDeselect(void)
{
  LastError = SPROX_Tcl_Deselect(Cid);
  return S_OK;
}

STDMETHODIMP CSpringProxActiveXCtl::get_TclData(BSTR * pVal)
{
  wchar_t buffer[2 * sizeof(TclDataRecv) + 1];

  toHEX(buffer, TclDataRecv, TclDataRecvLen);
  toBSTR(pVal, buffer);

  return S_OK;
}

STDMETHODIMP CSpringProxActiveXCtl::get_TclDataSW(LONG * SW)
{
  if (TclDataRecvLen >= 2)
  {
    *SW = TclDataRecv[TclDataRecvLen - 2];
    *SW *= 0x0100;
    *SW += TclDataRecv[TclDataRecvLen - 1];
  } else
    *SW = -1;

  return S_OK;
}

STDMETHODIMP CSpringProxActiveXCtl::put_TclData(BSTR newVal)
{
  size_t l;

  if (SysStringLen(newVal) > 2 * sizeof(TclDataSend))
    return Error(_T("Data is too long !"));

  if (!isHEX(newVal))
    return Error(_T("Data is not a valid hexadecimal value"));

  fromHEX(TclDataSend, newVal, sizeof(TclDataSend), &l);
  TclDataSendLen = (WORD) l;

  return S_OK;
}

STDMETHODIMP CSpringProxActiveXCtl::TclExchange(void)
{
  WORD rlen;

  if (!TclDataSendLen)
    return Error(_T("No data to exchange"));

  TclDataRecvLen = 0;

  rlen = sizeof(TclDataRecv);
  LastError =
    SPROX_Tcl_Exchange(Cid, TclDataSend, TclDataSendLen, TclDataRecv, &rlen);

  if (LastError != MI_OK)
    return Error(_T("T=CL exchange failed"));

  TclDataRecvLen = rlen;
  return S_OK;
}

STDMETHODIMP CSpringProxActiveXCtl::get_PageData(BSTR * pVal)
{
  wchar_t buffer[2 * 4 + 1];

  toHEX(buffer, MifULPageData, 4);
  toBSTR(pVal, buffer);

  return S_OK;
}

STDMETHODIMP CSpringProxActiveXCtl::get_PageDataTxt(BSTR * pVal)
{
  wchar_t buffer[4 + 1] = L"";
  int i;

  if (MifULPageDataOK == 4)
  {
    for (i=0; i<MifULPageDataOK; i++)
      buffer[i] = MifULPageData[i];
    buffer[MifULPageDataOK] = '\0';
  }
  toBSTR(pVal, buffer);

  return S_OK;
}

STDMETHODIMP CSpringProxActiveXCtl::put_PageData(BSTR newVal)
{
  if (SysStringLen(newVal) == 8)
  {
    MifULPageDataOK = 4;
  } else if (SysStringLen(newVal) == 0)
  {
    MifULPageDataOK = 0;
    return S_OK;
  } else  
  {
    MifULPageDataOK= 0;
    return Error(_T("Page data must be 4 bytes long"));
  }

  if (!isHEX(newVal))
  {
    MifULPageDataOK = 0;
    return Error(_T("Data is not a valid hexadecimal value"));
  }

  fromHEX(MifULPageData, newVal, 4, NULL);
  return S_OK;
}

STDMETHODIMP CSpringProxActiveXCtl::put_PageDataTxt(BSTR newVal)
{
  wchar_t *p;

  if (SysStringLen(newVal) > 4)
    return Error(_T("Page data must be 4 bytes long"));

  p = newVal;
  memset(MifULPageData, 0, 4);
  for (size_t i = 0; i < SysStringLen(newVal); i++)
    MifULPageData[i] = (BYTE) p[i];
    
  MifULPageDataOK = 4;

  return S_OK;
}


STDMETHODIMP CSpringProxActiveXCtl::get_BlockData(BSTR * pVal)
{
  wchar_t buffer[2 * 16 + 1];

  toHEX(buffer, MifBlockData, 16);
  toBSTR(pVal, buffer);

  return S_OK;
}

STDMETHODIMP CSpringProxActiveXCtl::get_BlockDataTxt(BSTR * pVal)
{
  wchar_t buffer[16 + 1] = L"";
  int i;

  if (MifBlockDataOK == 16)
  {
    for (i=0; i<MifBlockDataOK; i++)
      buffer[i] = MifBlockData[i];
    buffer[MifBlockDataOK] = '\0';
  }
  toBSTR(pVal, buffer);

  return S_OK;
}

STDMETHODIMP CSpringProxActiveXCtl::put_BlockData(BSTR newVal)
{
  if (SysStringLen(newVal) == 32)
  {
    MifBlockDataOK = 16;
  } else if (SysStringLen(newVal) == 0)
  {
    MifBlockDataOK = 0;
    return S_OK;
  } else  
  {
    MifBlockDataOK = 0;
    return Error(_T("Block data must be 16 bytes long"));
  }

  if (!isHEX(newVal))
  {
    MifBlockDataOK = 0;
    return Error(_T("Data is not a valid hexadecimal value"));
  }

  fromHEX(MifBlockData, newVal, 16, NULL);
  return S_OK;
}

STDMETHODIMP CSpringProxActiveXCtl::put_BlockDataTxt(BSTR newVal)
{
  wchar_t *p;

  if (SysStringLen(newVal) > 16)
    return Error(_T("Block data must be 16 bytes long"));

  p = newVal;
  memset(MifBlockData, 0, 16);
  for (size_t i = 0; i < SysStringLen(newVal); i++)
    MifBlockData[i] = (BYTE) p[i];
    
  MifBlockDataOK = 16;

  return S_OK;
}

STDMETHODIMP CSpringProxActiveXCtl::get_SectorData(BSTR * pVal)
{
  wchar_t buffer[2 * 240 + 1] = L"";

  if ((MifSectorDataOK == 240) || (MifSectorDataOK == 48))
  {
    toHEX(buffer, MifSectorData, MifSectorDataOK);
  }
  toBSTR(pVal, buffer);

  return S_OK;
}

STDMETHODIMP CSpringProxActiveXCtl::get_SectorDataTxt(BSTR * pVal)
{
  wchar_t buffer[240 + 1] = L"";
  int i;

  if ((MifSectorDataOK == 240) || (MifSectorDataOK == 48))
  {
    for (i=0; i<MifSectorDataOK; i++)
      buffer[i] = MifSectorData[i];
    buffer[MifSectorDataOK] = '\0';
  }
  toBSTR(pVal, buffer);

  return S_OK;
}

STDMETHODIMP CSpringProxActiveXCtl::put_SectorData(BSTR newVal)
{
  if (SysStringLen(newVal) == 96)
  {
    MifSectorDataOK = 48;
  } else if (SysStringLen(newVal) == 480)
  {
    MifSectorDataOK = 240;
  } else if (SysStringLen(newVal) == 0)
  {
    MifSectorDataOK = 0;
    return S_OK;
  } else  
  {
    MifSectorDataOK = 0;
    return Error(_T("Sector data must be 48 or 240 bytes long"));
  }

  if (!isHEX(newVal))
  {
    MifSectorDataOK = 0;
    return Error(_T("Data is not a valid hexadecimal value"));
  }

  fromHEX(MifSectorData, newVal, MifSectorDataOK, NULL);

  return S_OK;
}

STDMETHODIMP CSpringProxActiveXCtl::ReadPage(BYTE addr)
{
  MifULPageDataOK = 0;
  MifBlockDataOK  = 0;
  
  LastError = SPROX_MifRead(NULL, addr, MifBlockData);
  
  if (LastError != MI_OK)
    return Error(_T("Mifare UL read page failed"));
    
  MifBlockDataOK = 16;
  memcpy(MifULPageData, MifBlockData, 4);
  MifULPageDataOK = 4;

  return S_OK;
}

STDMETHODIMP CSpringProxActiveXCtl::WritePage(BYTE addr)
{
  if (MifULPageDataOK != 4)
    return Error(_T("You must populate the page data buffer first"));

  LastError = SPROX_MifWrite4(NULL, addr, MifULPageData);

  if (LastError != MI_OK)
    return Error(_T("Mifare UL write page failed"));

  return S_OK;

}

STDMETHODIMP CSpringProxActiveXCtl::ReadBlock(BYTE addr)
{
  MifBlockDataOK = 0;

  if (MifKeyReadLoaded)
  {
    LastError = SPROX_MifStReadBlock(NULL, addr, MifBlockData, MifKeyRead);
  } else
  {
    LastError = SPROX_MifStReadBlock(NULL, addr, MifBlockData, NULL);
  }

  if (LastError != MI_OK)
    return Error(_T("Mifare read block failed"));
    
  MifBlockDataOK = 16;

  return S_OK;
}

STDMETHODIMP CSpringProxActiveXCtl::WriteBlock(BYTE addr)
{
  if (MifBlockDataOK != 16)
    return Error(_T("You must populate the block data buffer first"));

  if (MifKeyWriteLoaded)
  {
    LastError = SPROX_MifStWriteBlock(NULL, addr, MifBlockData, MifKeyWrite);
  } else
  {
    LastError = SPROX_MifStWriteBlock(NULL, addr, MifBlockData, NULL);
  }

  if (LastError != MI_OK)
    return Error(_T("Mifare write block failed"));

  return S_OK;

}

STDMETHODIMP CSpringProxActiveXCtl::ReadSector(BYTE addr)
{
  MifSectorDataOK = 0;
  
  if (MifKeyReadLoaded)
  {
    LastError = SPROX_MifStReadSector(NULL, addr, MifSectorData, MifKeyRead);
  } else
  {
    LastError = SPROX_MifStReadSector(NULL, addr, MifSectorData, NULL);
  }

  if (LastError != MI_OK)
    return Error(_T("Mifare read sector failed"));

  MifSectorDataOK = (addr >= 32) ? 240 : 48;

  return S_OK;
}

STDMETHODIMP CSpringProxActiveXCtl::WriteSector(BYTE addr)
{
  if (MifSectorDataOK == 48)
  {
    if (addr >= 32)
    {
      return Error(_T("Check sector data buffer (size must be 240)"));    
    }
  } else
  if (MifSectorDataOK == 240)
  {
    if (addr < 32)
    {
      return Error(_T("Check sector data buffer (size must be 48)"));  
    }
  } else  
  {  
    return Error(_T("You must populate the sector data buffer first"));
  }

  if (MifKeyWriteLoaded)
  {
    LastError =
      SPROX_MifStWriteSector(NULL, addr, MifSectorData, MifKeyWrite);
  } else
  {
    LastError = SPROX_MifStWriteSector(NULL, addr, MifSectorData, NULL);
  }

  if (LastError != MI_OK)
    return Error(_T("Mifare write sector failed"));

  return S_OK;
}

STDMETHODIMP CSpringProxActiveXCtl::FormatSector(BYTE addr, BSTR KeyA, BSTR KeyB,
                                          BYTE ac0, BYTE ac1, BYTE ac2)
{
  return WriteSectorTrailer(addr, KeyA, KeyB, ac0, ac1, ac2);
}

STDMETHODIMP CSpringProxActiveXCtl::WriteSectorTrailer(BYTE addr, BSTR KeyA,
                                                BSTR KeyB, BYTE ac0, BYTE ac1,
                                                BYTE ac2)
{
  BYTE keyA[6];
  BYTE keyB[6];
  BYTE *keyW = MifKeyWriteLoaded ? MifKeyWrite : NULL;

  if (!getKey(keyA, KeyA))
  {
    return Error(_T("Invalid value for key A"));
  }
  if (!getKey(keyB, KeyB))
  {
    return Error(_T("Invalid value for key B"));
  }

  LastError =
    SPROX_MifStUpdateAccessBlock(NULL, addr, keyW, keyA, keyB, ac0, ac1, ac2,
                                 ACC_AUTH_NORMAL);

  if (LastError != MI_OK)
    return Error(_T("Write sector trailer failed"));

  return S_OK;
}

STDMETHODIMP CSpringProxActiveXCtl::get_LastAuthKey(BSTR * pVal)
{
  wchar_t buffer[256];
  BYTE key;

  LastError = SPROX_MifLastAuthKey(&key);
  if (LastError != MI_OK)
    return Error(_T("MifLastAuthKey failed"));

  _stprintf(buffer, _T("%d"), key);
  toBSTR(pVal, buffer);

  return S_OK;

}

STDMETHODIMP CSpringProxActiveXCtl::put_MifKeyRead(BSTR newVal)
{  
  MifKeyReadLoaded = FALSE;
  
  if (SysStringLen(newVal) > 0)
  {
    if (!getKey(MifKeyRead, newVal))
      return Error(_T("Invalid value for key"));
      
    MifKeyReadLoaded = TRUE;
  }

  return S_OK;
}

STDMETHODIMP CSpringProxActiveXCtl::put_MifKeyWrite(BSTR newVal)
{
  MifKeyWriteLoaded = FALSE;
  
  if (SysStringLen(newVal) > 0)
  {
    if (!getKey(MifKeyWrite, newVal))
      return Error(_T("Invalid value for key"));
      
    MifKeyWriteLoaded = TRUE;
  }

  return S_OK;
}

STDMETHODIMP CSpringProxActiveXCtl::LoadKey(BOOL E2, BOOL B, BYTE index, BSTR newVal)
{
  BYTE key[6];

  if (!getKey(key, newVal))
    return Error(_T("Invalid value for key"));

  if (B)
    LastError = SPROX_MifLoadKey(E2, 'B', index, key);
  else
    LastError = SPROX_MifLoadKey(E2, 'A', index, key);

  if (LastError != MI_OK)
    return Error(_T("Failed to load the key"));
  return S_OK;
}

void CSpringProxActiveXCtl::toBSTR(BSTR * dst, const wchar_t * src)
{
  wchar_t *p;

  if (src == NULL)
    return;
  if (dst == NULL)
    return;

  size_t l = wcslen(src);

  if (*dst != NULL)
    SysFreeString(*dst);

  p = SysAllocStringLen(NULL, (UINT) (l + 1));

  for (size_t i = 0; i <= l; i++)
    p[i] = src[i];

  *dst = p;
}

void CSpringProxActiveXCtl::fromBSTR(wchar_t * dst, const BSTR src)
{
  if (src == NULL)
    return;
  if (dst == NULL)
    return;

  int l1 = SysStringLen(src);
  int l2 =
    WideCharToMultiByte(CP_ACP, 0, src, l1, (LPSTR) dst, l1, NULL, NULL);

  dst[min(l1, l2)] = '\0';
}

BOOL CSpringProxActiveXCtl::isHEX(const wchar_t * src)
{
  if (src == NULL)
    return FALSE;

  size_t l = wcslen(src);

  if (l % 2)
    return FALSE;

  for (size_t i = 0; i < l; i++)
  {
    if ((src[i] >= '0') && (src[i] <= '9'))
      continue;
    if ((src[i] >= 'A') && (src[i] <= 'F'))
      continue;
    if ((src[i] >= 'a') && (src[i] <= 'f'))
      continue;
    return FALSE;
  }

  return TRUE;
}

void CSpringProxActiveXCtl::toHEX(wchar_t * dst, const BYTE * src, size_t sz)
{
  if (src == NULL)
    return;
  if (dst == NULL)
    return;

  for (size_t i = 0; i < sz; i++)
  {
    _stprintf(&dst[2 * i], _T("%02X"), src[i]);
  }
}

void CSpringProxActiveXCtl::fromHEX(BYTE * dst, const wchar_t * src, size_t max_sz,
                             size_t * act_sz)
{
  if (src == NULL)
    return;

  size_t l = wcslen(src);
  size_t i;
  BYTE b;

  for (i = 0; i < l; i++)
  {
    if ((i / 2) > max_sz)
      break;

    if ((src[i] >= '0') && (src[i] <= '9'))
    {
      b = (BYTE) (src[i] - '0');
    } else if ((src[i] >= 'A') && (src[i] <= 'F'))
    {
      b = (BYTE) (src[i] + 10 - 'A');
    } else if ((src[i] >= 'a') && (src[i] <= 'f'))
    {
      b = (BYTE) (src[i] + 10 - 'a');
    } else
      return;

    if (i % 2)
    {
      dst[i / 2] |= b;
    } else
    {
      dst[i / 2] = b << 4;
    }
  }

  if (act_sz != NULL)
    *act_sz = i / 2;
}

BOOL CSpringProxActiveXCtl::getKey(BYTE * dst, BSTR src)
{
  wchar_t buffer[12 + 1];
  int i;

  if (SysStringLen(src) == 12)
  {
    for (i=0; i<12; i++)
      buffer[i] = src[i];
    buffer[12] = '\0';  

    if (!isHEX(buffer))
      return FALSE;

    fromHEX(dst, buffer, 6, NULL);
    return TRUE;

  } else if (SysStringLen(src) == 6)
  {
    for (i=0; i<6; i++)
      dst[i] = (BYTE) src[i];
    return TRUE;
  }

  return FALSE;
}

