#include "cardappz.h"
#include <assert.h>
#include <stdio.h>
#include <conio.h>

static int getEndianess()
{
  unsigned long x = 0x00010203;
  unsigned char le[4] = { 0, 1, 2, 3 }; /* 0x00010203 */
  unsigned char be[4] = { 3, 2, 1, 0 };
  unsigned char *pos;           /*  32-bit LE long starts here, for 
                                   architectures with > 32 bit longs */

  static int result = 0;

  if (result)
    return result;

  assert(sizeof(x) >= 4);
  pos = (unsigned char *) &x;
  pos += sizeof(x) - 4;
  if (!memcmp(le, pos, 4))
  {
    result = 1;                 /* Little endian */
  } else if (!memcmp(be, &x, 4))
  {
    result = 2;                 /* Big endian */
  }
  return result;
}


unsigned short CtoHs(unsigned short v)
{
  if (getEndianess() == 1)
  {
    /* Little endian : we must revert the short value */
    register unsigned short t;

    t = (v >> 8) & 0x00FF;
    t |= (v << 8) & 0xFF00;
    return t;
  }
  return v;
}

unsigned short HtoCs(unsigned short v)
{
  if (getEndianess() == 1)
  {
    /* Little endian : we must revert the short value */
    register unsigned short t;

    t = (v >> 8) & 0x00FF;
    t |= (v << 8) & 0xFF00;
    return t;
  }
  return v;
}

unsigned long CtoHl(unsigned long v)
{
  if (getEndianess() == 1)
  {
    /* Little endian : we must revert the long value */
    register unsigned long t;

    t = (v >> 24) & 0x000000FF;
    t |= (v >> 8) & 0x0000FF00;
    t |= (v << 8) & 0x00FF0000;
    t |= (v << 24) & 0xFF000000;
    return t;
  }
  return v;
}

unsigned long HtoCl(unsigned long v)
{
  if (getEndianess() == 1)
  {
    /* Little endian : we must revert the long value */
    register unsigned long t;

    t = (v >> 24) & 0x000000FF;
    t |= (v >> 8) & 0x0000FF00;
    t |= (v << 8) & 0x00FF0000;
    t |= (v << 24) & 0xFF000000;
    return t;
  }
  return v;
}

unsigned char gethexq(char c)
{
  if ((c >= '0') && (c <= '9'))
    return (c - '0');
  if ((c >= 'A') && (c <= 'F'))
    return (c - 'A' + 10);
  if ((c >= 'a') && (c <= 'f'))
    return (c - 'a' + 10);
  return 0xCC;
}

void gethex(char *src, unsigned char *dst, size_t dstsize)
{
  size_t l, i;

  assert(dst != NULL);

  memset(dst, 0xCC, dstsize);
  if (src != NULL)
  {
    l = strlen(src);
    for (i = 0; i < l; i += 2)
    {
      if ((i / 2) >= dstsize)
        return;
      dst[i / 2] = gethexq(src[i]) << 4;
      dst[i / 2] |= gethexq(src[i + 1]);
    }
  }
}

int confirm(char *prompt, int def)
{
  int c;

  for (;;)
  {
    printf(prompt);
    if (def)
    {
      printf(" [YES/no] ? ");
    } else
    {
      printf(" [yes/NO] ? ");
    }
    c = getch();
    c = toupper(c);
    printf("\n");
    switch (c)
    {
      case 'Y':
        return 1;
      case 'N':
        return 0;
      case '\0':
      case '\n':
      case '\r':
        return def;
    }
  }
}

void pause(void)
{
  printf("-- (Hit a key)");
  getch();
  printf("\n");
}
