#ifndef _CARDAPPZ_TYPES_H_
#define _CARDAPPZ_TYPES_H_

/* We use the time_t type */
#include <time.h>

/* We must have structure members aligned on byte boundary */
#pragma pack(1)

/* ADT will be define later because each module may provide its own typedefs or defines */
typedef struct cardappz_ctx_st CARDAPPZ_CTX_ST;

/*
 * Corporate ID application
 * ------------------------
 */
typedef struct
{
  char LastName[32];
  char FirstName[32];
  char Company[24];
  char Title[24];
  unsigned char BirthDate_Year;
  unsigned char BirthDate_Month;
  unsigned char BirthDate_Day;
  unsigned char rfu[13];  
} CORPID_IDENTITY_ST;

#define CORPID_FINGERPRINTS_MAX_SZ 170
#define CORPID_PICTURE_MAX_SZ 4096

typedef struct
{
  unsigned char UserID[4];
  unsigned char StartDate_Year;
  unsigned char StartDate_Month;
  unsigned char StartDate_Day;
  unsigned char EndDate_Year;
  unsigned char EndDate_Month;
  unsigned char EndDate_Day;
  unsigned char FingerPrintsType;
  unsigned char PictureType;
  unsigned short PictureSize;
  unsigned char rfu[2];
  unsigned char FastSignature[8];
  unsigned char ShortSignature[8]; 
} CORPID_OPTIONSSIGN_ST;

/*
 * Electronic purse application
 * ----------------------------
 */

typedef struct
{
  unsigned char TimeStamp_Year;
  unsigned char TimeStamp_Month;
  unsigned char TimeStamp_Day;
  unsigned char TimeStamp_Hour;
  unsigned char TimeStamp_Min;
  unsigned char Issuer[4];
  unsigned char Payment;
  unsigned short Credit;
  unsigned long NewBalance;
} PURSE_RELOAD_ST;

typedef struct
{
  unsigned char TimeStamp_Year;
  unsigned char TimeStamp_Month;
  unsigned char TimeStamp_Day;
  unsigned char TimeStamp_Hour;
  unsigned char TimeStamp_Min;
  unsigned char Merchant[4];
  unsigned char Vendor;
  unsigned short Debit;
  unsigned char VendorData[4];
} PURSE_PURCHASE_ST;

/*
 * Public transportation application
 * ---------------------------------
 */


/*
 * Data manipulation functions
 * ---------------------------
 */
void Time2TimeStamp(time_t src, unsigned char *y, unsigned char *m, unsigned char *d, unsigned char *h, unsigned char *n);
void TimeStamp2Time(time_t dst, unsigned char y, unsigned char m, unsigned char d, unsigned char h, unsigned char n);

unsigned short CtoHs(unsigned short v);
unsigned short HtoCs(unsigned short v);
unsigned long  CtoHl(unsigned long v);
unsigned long  HtoCl(unsigned long v);

void gethex(char *src, unsigned char *dst, size_t dstsize);

int confirm(char *prompt, int def);
void pause(void);

#endif
